C @(#)[fits2mef.f] Write 'calstis'-compliant MEF file from a series of flat
C @(#)FITS files, part of the same STIS data set, from which Side-2 pattern
C @(#)has been removed (using 'autofilet').     R.A. Jansen -- Jul  5, 2010
C @(#)
C @(#) Purpose: Merge a series of individual flat FITS files, that were ex-
C @(#)   tracted  from an original STIS data set  and from which the Side-2
C @(#)   herring-bone pattern noise was removed with the IDL implementation
C @(#)   of 'autofilet', with header units from that original STIS data set
C @(#)   into a Multi-Extension FITS (MEF) file that is acceptable as input
C @(#)   to 'calstis' .  The primary header of the output MEF file includes
C @(#)   a new keyword "HPATCORR", before "DQICORR", with value 'COMPLETE'.
C @(#)   Each [SCI] extension header will include a new header section that
C @(#)   summarizes the characteristics of the detected herringbone pattern
C @(#)   and whether or not that pattern was deemed valid. If not, then the
C @(#)   frame in question will hold a 32-bit (floating point real) copy of
C @(#)   the original 16-bit integer valued frame (fail-safe behavior).
C @(#)
C @(#) Dependencies:  requires multiple routines from the  'rjgen'  and the
C @(#)   'rjfits'  libraries, both by the author.
C @(#)
C @(#) Compilation instructions:
C @(#)   (1)  make    (to compile the 'rjgen' and 'rjfits' libraries)
C @(#)   (2)  /usr/bin/g77 -ff77 -pedantic -Wall -Werror -W \
C @(#)        -ffortran-bounds-check -o fits2mef fits2mef.f \
C @(#)        -Llib -lrjfits -lrjgen
C @(#)
C @(#) Syntax (example):
C @(#) $ set imr="o69531010"         (or:  set imr="o8i105050_wav")
C @(#) $ \cp -p $psub$imr"_rwc"*".fits" .
C @(#) $ grep $imr $tabpath/_PATTERNFQ_200* | awk -F: '{print $2}' | \
C @(#)   sed 's:_wav_rwc:+wav_rwc:g' | sed 's:_: :g' | sed 's:.fits::g' | \
C @(#)   awk '{if($7==0){print $3",PSUB/"$1"_rwc_"$3".fits,"$4","$5","$6","$7} \
C @(#)        else {print $3",FAIL/"$1"_raw_"$3".fits,"$4","$5","$6","$7} }' | \
C @(#)   sed 's:+:_:g' | sed 's:*:1:g' > $imr"_rwc.inp"
C @(#) $ fits2mef $imr"_raw.fits" $orig$imr"_raw.fits" $imr.inp
C @(#)
C @(#) Note: this is an initial but working version for unbinned full-frame
C @(#)   Side-2 STIS/CCD data.  It is likely riddled with bugs (eg, missing
C @(#)   tests and uncaptured exceptions, hard-wired variable values, etc.)
C @(#)   if run on any other kind of data.  Beware!
C @(#)
C @(#) Revision history:
C @(#)   Apr 16, 2010 [RAJ] -- Initial outline of this program.
C @(#)   Jun 27, 2010 [RAJ] -- (Finally got back to this again...)
C @(#)   Jul 05, 2010 [RAJ] -- Version 1.0, used to merge data delivered to
C @(#)                         the HST Archive as part of HST-AR-11258.
C @(#) --------------------------------------------------------------------

      PROGRAM fits2mef

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Learn about maximum image dimension, maximum axis lengths, maximum
C     number of FITS extensions, and maximum number of header records...
      INCLUDE	'buffers.inc'

C --- Declaration of program variables...
C     -- Integer error status and verbosity level...
      INTEGER	ierr, idbg
      CHARACTER dbid*19

C     -- File I/O variables and boolean switch...
      CHARACTER*80 omef, imef, tfil, ifits(MAXEXT)
      CHARACTER*2880 block, tmpblk
      INTEGER	olen, ilen, tlen, nf, flen(MAXEXT), iext(MAXEXT)
      INTEGER	iblk, oblk, jblk
      LOGICAL	readonly

C     -- Memory I/O variables...
      CHARACTER*80 crec, ccom, sval, ctmp, ohdr(MAXREC)
      CHARACTER*80 spver, sdate
      CHARACTER*8 ckey, cfmt
      REAL	hparm(MAXEXT,4), rval
      INTEGER	nrec, ival, jrec, ndu, nblk
      INTEGER	bpp, nxtd, naxes, axlen(MAXAX), gcnt, pcnt, slen
C     -- Boolean switches...
      LOGICAL	lfix, lold, ltrnc
C     -- Logical flags...
      LOGICAL	last, lcom, lok

C     Other variables (mostly loop counters)...
      INTEGER	clen, i, j, k, m, n, nskip

C --------------------------   M  A  I  N   ----------------------------

C --- Initialize... 
      ierr   = 0
      idbg   = 0
      dbid   = 'DEBUG(fits2mef):   '
      lfix   = .TRUE.
      lold   = .TRUE.
      ltrnc  = .TRUE.
      cfmt   = '        '
      gcnt   = 1
      pcnt   = 0
      naxes  = 0
      axlen(1) = 0
      axlen(2) = 0

C --- Read, parse, and verify the command-line input: if successful, we
C     will have read the name of the output MEF file 'omef' (of length
C     'olen' characters), the name of the original STIS data set 'imef'
C     (of length 'ilen' characters), and the name of an input text file
C     'tfil' (of length 'tlen' characters), and optionally a verbosity
C     modifier...
      IF (idbg.GT.0) WRITE(*,*) dbid//'calling "clinput"'
      CALL clinput (omef, olen, imef, ilen, tfil, tlen, idbg, ierr)
      IF (ierr.EQ.-1) CALL manual()
      IF (ierr.NE.0) GOTO 9
      WRITE(*,'(A)') 'FITS2MEF: "'//imef(1:ilen)//',@'//tfil(1:tlen)//
     +        '" --> "'//omef(1:olen)//'" ...'

C --- Read, parse, and verify the input text file: if successful, return
C     an 'nf'-element array of names of the FITS files to merge into the
C     output MEF file, and a floating-point array 'hparm(nf,4)' of the
C     inferred pattern-noise parameters and pass/fail flag...
      IF (idbg.GT.0) WRITE(*,*) dbid//'calling "flinput"'
      CALL flinput (tfil,tlen, nf, ifits, flen, iext, hparm, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      IF (idbg.GT.0) THEN
         DO i=1,nf
            WRITE(*,*) dbid//'input FITS file #',i,' = "'//
     +         ifits(i)(1:flen(i))//'" (iext=',iext(i),')'
         ENDDO
      ENDIF


C --- Try and open the named output MEF file for direct read-write
C     access at the standard logical device unit reserved for output to
C     disk file...
      IF (idbg.GT.0) WRITE(*,*) dbid//'calling "rjf_open" (RW)'
      IF (idbg.GT.0) WRITE(*,*) dbid//' file="'//omef(1:olen)//'"'
      readonly = .FALSE.
      CALL rjf_open (STDOFIL, omef, readonly, idbg, ierr)
      IF (ierr.NE.0) GOTO 9


C --- Open the original STIS dataset (MEF file) for direct read-only
C     access at the standard logical device unit reserved for input from
C     disk file and try and copy the primary header unit to the open
C     output MEF file, while inserting the new HPATCORR header record...
      IF (idbg.GT.0) WRITE(*,*) dbid//'calling "rjf_open" (RO)'
      IF (idbg.GT.0) WRITE(*,*) dbid//' file="'//imef(1:ilen)//'"'
      readonly = .TRUE.
      CALL rjf_open (STDIFIL, imef, readonly, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Try to read the first 2880-byte block of the Primary Header Unit..
      last = .FALSE.
      nrec = 0
      iblk = 0
1     iblk = iblk + 1
      CALL rjfghblk (STDIFIL, iblk, block, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Parse this block into 36 individual 80-char header records...
      DO i = 1,36
C        Read the next record from the 2880-byte block...
         IF (.NOT.last) THEN
           CALL rjfghrec (block, i, crec, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           nrec = nrec + 1
           IF (idbg.GT.1) WRITE(*,'(1x,(A),I3,(A))') dbid//'record #',
     +                  nrec,':  "'//crec(1:clen(crec))//'"'

C          Parse the keyword from the record; search for BITPIX, NAXIS*,
C          NEXTEND, DQICORR and END ...
           CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
           IF (ierr.NE.0) GOTO 9

           ival = -1
           IF (ckey(1:j).EQ.'END') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'detected END keyword; last = T'
             lcom = .FALSE.
             last = .TRUE.
           ELSE IF (ckey(1:j).EQ.'BITPIX') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
                bpp = ival
                IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected BITPIX keyword; bpp =',bpp
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NAXIS') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
                naxes = ival
                IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS keyword; naxes =',naxes
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NAXIS1') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
                axlen(1) = ival
                IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS1 keyword; naxis1 =',axlen(1)
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NAXIS2') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
                axlen(2) = ival
                IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS2 keyword; naxis2 =',axlen(2)
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NEXTEND') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
                nxtd = ival
                IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NEXTEND keyword; nxtd =',nxtd
             ENDIF
           ELSE IF (ckey(1:j).EQ.'DQICORR') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'detected DQICORR keyword; inserting HPATCORR before it'
C            Insert 'HPATCORR' header keyword with value 'COMPLETE'...
             CALL cinit(ctmp)
             ckey = 'HPATCORR'
             CALL rjfpkwrd (ctmp, ckey, idbg, ierr)
             sval = 'COMPLETE'
             CALL rjfpkvals(ctmp, sval, lold, idbg, ierr)
             ccom = 'remove herring-bone pattern noise'
             CALL rjfpkcom (ctmp, ccom, ltrnc, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
           ELSE
C            Some other header keyword detected...
             CONTINUE
           ENDIF
           IF (ierr.NE.0) GOTO 9

C          Append the record read to the output header array...
           ohdr(nrec) = crec
         ENDIF
      ENDDO

C     Try and read the next 2880-byte block from the primary header...
      IF (.NOT.last) GOTO 1
      IF (idbg.GT.0) WRITE(*,*) dbid//' --> PHU has ',nrec,' keywords.'

C     Write the modified Primary Header to the output MEF file, one 2880
C     byte block at a time...
      IF (idbg.GT.0) WRITE(*,*) dbid//
     +   '  writing PHU to output MEF file...'
      j    = 0
      oblk = 1
      CALL cinit(tmpblk)
      DO i = 1,nrec
         j = j + 1
         IF (j.LE.36) THEN
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ELSE
C           Write the 2880-byte block to the output MEF file...
            CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
            IF (ierr.NE.0) GOTO 9
C           Re-initialize the 2880-byte block buffer...
            j = 1
            oblk = oblk + 1
            CALL cinit(tmpblk)
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ENDIF
      ENDDO

C     Write the partially populated (remainder is blank) or exactly
C     filled 2880-byte block to the output MEF file...
      CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      WRITE(*,*)' wrote PHU to output MEF file (',oblk,' blocks )'

C --- Loop over all extension headers in the original STIS dataset (MEF)
C     file, copy the (minimally modified) content to the output MEF file
C     and substitute successfully cleaned frames or 32-bit copies of the
C     original frames for the corresponding data units...

C  -- Begin of loop over the extensions
      n = 0
2     n = n + 1

C     Try and open the next input flat FITS file, containing either a
C     pattern-cleaned frame or a 32-bit copy of the original frame, for
C     direct read-only access at the standard logical alternate device
C     unit reserved for input from disk file...
      IF (idbg.GT.0) WRITE(*,*)dbid//'calling "rjf_open" (RO)'
      IF (idbg.GT.0) WRITE(*,*)dbid//' file="'//ifits(n)(1:flen(n))//'"'
      readonly = .TRUE.
      CALL rjf_open (STDIFIL2, ifits(n), readonly, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      IF (iext(n).NE.((3*n)-2)) THEN
         WRITE(*,*) dbid//' mismatch of input frame list and extension'
         ierr = 1
      ENDIF
      IF (ierr.NE.0) GOTO 9

      last = .FALSE.
      jrec = 0
      jblk = 0
3     jblk = jblk + 1
      CALL rjfghblk (STDIFIL2, jblk, block, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Parse this block into 36 individual 80-char header records...
      DO i = 1,36
C        Read the next record from the 2880-byte block...
         IF (.NOT.last) THEN
           CALL rjfghrec (block, i, crec, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           jrec = jrec + 1
           IF (idbg.GT.1) WRITE(*,'(1x,(A),I3,(A))') dbid//'record #',
     +                  jrec,':  "'//crec(1:clen(crec))//'"'

C          Parse each record and search for ORIGIN, DATE, and END ...
           CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           IF (ckey(1:j).EQ.'END') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'detected END keyword; last = T'
             lcom = .FALSE.
             last = .TRUE.
           ELSE IF (ckey(1:j).EQ.'ORIGIN') THEN
             CALL rjfgkvals (crec, sval, slen, idbg, ierr)
             IF (ierr.EQ.0) THEN
               spver = sval
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected ORIGIN = "'//spver(1:slen)//'"'
             ENDIF
           ELSE IF (ckey(1:j).EQ.'DATE') THEN
             CALL rjfgkvals (crec, sval, slen, idbg, ierr)
             IF (ierr.EQ.0) THEN
               sdate = sval
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected DATE = "'//sdate(1:slen)//'"'
             ENDIF
           ELSE
C            Some other header keyword detected...
             CONTINUE
           ENDIF
           IF (ierr.NE.0) GOTO 9
         ENDIF
      ENDDO
C     Try and read the next 2880-byte block from the primary header...
      IF (.NOT.last) GOTO 3
      IF (idbg.GT.0) WRITE(*,*) dbid//' --> HU has ',jrec,' keywords.'


C     In the original STIS dataset MEF file, skip to the next extension
C     header unit (always immediately following the previous dummy DQ
C     extension)... 
      nskip = 0
      IF (idbg.GT.0) WRITE(*,*) dbid//' start of extension HDU at '//
     +   '2880b-block #',(iblk+1)

C     Copy the extension header to the open output MEF file, but change
C     the value of BITPIX from 16 to -32, that of BZERO from 32768.0 to
C     0.0, and append a header section on the properties of the removed
C     noise pattern...
      last = .FALSE.
      nrec = 0
4     iblk = iblk + 1
      CALL rjfghblk (STDIFIL, iblk, block, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Parse this block into 36 individual 80-char header records...
      DO i = 1,36
C        Read the next record from the 2880-byte block...
         IF (.NOT.last) THEN
           CALL rjfghrec (block, i, crec, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           nrec = nrec + 1
           IF (idbg.GT.1) WRITE(*,'(1x,(A),I3,(A))') dbid//'record #',
     +                  nrec,':  "'//crec(1:clen(crec))//'"'

C          Parse the keyword from the record; search for BITPIX, NAXIS*,
C          BZERO, and END..
           CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           ival = -1
           IF (ckey(1:j).EQ.'BITPIX') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
               bpp = ival
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected BITPIX keyword; bpp =',bpp
             ENDIF
             ival = -32
             CALL rjfpkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'updated value of BITPIX to ',ival
           ELSE IF (ckey(1:j).EQ.'NAXIS') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
               naxes = ival
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS keyword; naxes =',naxes
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NAXIS1') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
               axlen(1) = ival
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS1 keyword; naxis1 =',axlen(1)
             ENDIF
           ELSE IF (ckey(1:j).EQ.'NAXIS2') THEN
             CALL rjfgkvali (crec, ival, idbg, ierr)
             IF (ierr.EQ.0) THEN
               axlen(2) = ival
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected NAXIS2 keyword; naxis2 =',axlen(2)
             ENDIF
           ELSE IF (ckey(1:j).EQ.'BZERO') THEN
             CALL rjfgkvale (crec, rval, idbg, ierr)
             IF (ierr.EQ.0) THEN
               IF (idbg.GT.0) WRITE(*,*) dbid//
     +            'detected BZERO keyword; bzero =',rval
             ENDIF
             crec(24:28) = '    0'
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'updated value of BZERO to 0.0'
           ELSE IF (ckey(1:j).EQ.'END') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//'detected END keyword '//
     +         '(last = T); inserting HPAT* before it'
C            Insert 'HPAT*' section recording pattern characteristics...
             CALL cinit(ctmp)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             CALL cinit(ckey)
             CALL cinit(ccom)
             ccom='     / HERRINGBONE PATTERN NOISE REMOVAL INFORMATION'
             CALL rjfpcomm(ctmp, ckey, ccom, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             CALL cinit(ctmp)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'inserted HPAT section heading'

             CALL cinit(ctmp)
             CALL cinit(ccom)
             ckey = 'HPATVER'
             ccom = 'program version'
             CALL rjfpkwrd (ctmp, ckey, idbg, ierr)
             CALL rjfpkvals(ctmp, spver, lold, idbg, ierr)
             CALL rjfpkcom (ctmp, ccom, ltrnc, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'inserted HPATVER keyword'

             CALL cinit(ctmp)
             CALL cinit(ccom)
             ckey = 'HPATDATE'
             ccom = 'date program was run'
             CALL rjfpkwrd (ctmp, ckey, idbg, ierr)
             CALL rjfpkvals(ctmp, sdate, lold, idbg, ierr)
             CALL rjfpkcom (ctmp, ccom, ltrnc, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'inserted HPATDATE keyword'

             CALL cinit(ctmp)
             CALL cinit(sval)
             CALL cinit(ccom)
             ckey = 'HPATPARS'
             WRITE(sval,'(A5,F7.1,A11,F6.4,A10,F7.5)') 'freq=',
     +            hparm(n,1),' Hz, width=',hparm(n,2),' Hz, pmax=',
     +            hparm(n,3)
             ccom = 'detected pattern'
             CALL rjfpkwrd (ctmp, ckey, idbg, ierr)
             CALL rjfpkvals(ctmp, sval, lold, idbg, ierr)
             CALL rjfpkcom (ctmp, ccom, ltrnc, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'inserted HPATPARS keyword'

             CALL cinit(ctmp)
             CALL cinit(ccom)
             ckey = 'HPATPASS'
             lok = (hparm(n,4).EQ.0.0)
             ccom = 'success (T), failed (F; frame is original*1.0)'
             CALL rjfpkwrd (ctmp, ckey, idbg, ierr)
             CALL rjfpkvall(ctmp, lok, idbg, ierr)
             CALL rjfpkcom (ctmp, ccom, ltrnc, idbg, ierr)
             ohdr(nrec) = ctmp
             nrec = nrec + 1
             IF (ierr.EQ.0.AND.idbg.GT.0) WRITE(*,*) dbid//
     +         'inserted HPATPASS keyword'

             lcom = .FALSE.
             last = .TRUE.
           ELSE
C            Some other header keyword detected...
             CONTINUE
           ENDIF
           IF (ierr.NE.0) GOTO 9

C          Append the record read to the output header array...
           ohdr(nrec) = crec
        ENDIF
      ENDDO

C     Try and read the next 2880-byte block from the extension header...
      IF (.NOT.last) GOTO 4
      IF (MOD(nrec,36).EQ.0) nrec = nrec - 1
      IF (idbg.GT.0) WRITE(*,*) dbid//' --> XHU has ',nrec,' keywords.'

C     Write the modified Extension Header to the output MEF file, one
C     2880-byte block at a time...
      IF (idbg.GT.0) WRITE(*,*) dbid//' writing XHU #',(3*n-2),
     +   ' to output MEF file...'
      j    = 0
      nblk = oblk
      oblk = oblk + 1
      CALL cinit(tmpblk)
      DO i = 1,nrec
         j = j + 1
         IF (j.LE.36) THEN
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ELSE
C           Write the 2880-byte block to the output MEF file...
            CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
            IF (ierr.NE.0) GOTO 9
C           Re-initialize the 2880-byte block buffer...
            j = 1
            oblk = oblk + 1
            CALL cinit(tmpblk)
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ENDIF
      ENDDO

C     Write the partially populated (remainder is blank) or exactly
C     filled 2880-byte block to the output MEF file...
      CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      WRITE(*,*)' wrote XHU #',(3*n-2),' to output MEF file (',
     +   (oblk-nblk),' blocks )'


C     Copy the Data Unit from the current flat FITS frame to the output
C     MEF file, one 2880-byte block at a time...
      IF (idbg.GT.0) WRITE(*,*) dbid//' writing XDU #',(3*n-2),
     +   ' to output MEF file...'
      rval = (32/8)*gcnt*(pcnt + axlen(1)*axlen(2))/2880.
      ndu  = INT(rval)
      IF ((rval-ndu).GT.0.0) ndu = ndu + 1
      IF (idbg.GT.0) WRITE(*,*) dbid//'copying blocks #',(jblk+1),
     +   ' thru #',(jblk+ndu)
      DO i = (jblk+1),(jblk+ndu),1
         CALL rjfghblk (STDIFIL2, i, block, idbg, ierr)
         IF (ierr.NE.0) GOTO 9
         oblk = oblk + 1
         CALL rjfphblk (block, STDOFIL, oblk, idbg, ierr)
         IF (ierr.NE.0) GOTO 9
      ENDDO
      WRITE(*,*)' wrote XDU #',(3*n-2),' to output MEF file (',ndu,
     +   ' blocks )'

C     Close the open input flat FITS file...
      IF (idbg.GT.0) WRITE(*,*) dbid//
     +   'calling "rjf_close" (closing "'//ifits(n)(1:flen(n))//'")'
      CALL rjf_close(STDIFIL2, idbg, ierr)
      IF (ierr.NE.0) GOTO 9


C     In the original STIS dataset MEF file, skip to the next extension
C     header unit...
      IF (naxes.EQ.0) THEN
C     -- Next HDU will start immediately following the present HDU, so
C        we don't have to skip over any binary data...
         nskip = 0
      ELSE
C     -- We need to skip over the binary data unit to get to the start
C        of the next HDU...
         nskip = 1
         DO k=1,naxes
            nskip = nskip*axlen(k)
         ENDDO
         nskip = (ABS(bpp)/8)*gcnt*(pcnt + nskip)
      ENDIF
C     Starting address of the first 2880-byte block of the next HDU...
      IF (nskip.GT.0) iblk = iblk + (INT(nskip/2880) + 1)
      IF (idbg.GT.0) THEN
         WRITE(*,*) dbid//' Data Unit in "'//imef(1:ilen)//'" spans ',
     +     nskip,' bytes'
         WRITE(*,*) dbid//' start of extension HDU at 2880b-blo'//
     +     'ck #',(iblk+1)
      ENDIF


C     Copy the dummy ERR extension in original STIS dataset MEF file...
      last = .FALSE.
      nrec = 0
5     iblk = iblk + 1
      CALL rjfghblk (STDIFIL, iblk, block, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Parse this block into 36 individual 80-char header records...
      DO i = 1,36
C        Read the next record from the 2880-byte block...
         IF (.NOT.last) THEN
           CALL rjfghrec (block, i, crec, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           nrec = nrec + 1
           IF (idbg.GT.1) WRITE(*,'(1x,(A),I3,(A))') dbid//'record #',
     +                  nrec,':  "'//crec(1:clen(crec))//'"'

C          Parse the keyword from the record; search for END...
           CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           IF (ckey(1:j).EQ.'END') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'detected END keyword; last = T'
             lcom = .FALSE.
             last = .TRUE.
           ELSE
C            Some other header keyword detected...
             CONTINUE
           ENDIF
           IF (ierr.NE.0) GOTO 9

C          Append the record read to the output header array...
           ohdr(nrec) = crec
         ENDIF
      ENDDO

C     Try and read the next 2880-byte block from the extension header...
      IF (.NOT.last) GOTO 5
      IF (MOD(nrec,36).EQ.0) nrec = nrec - 1
      IF (idbg.GT.0) WRITE(*,*) dbid//' --> XHU has ',nrec,' keywords.'

C     Write the dummy extension (just a HU) to the output MEF file, one
C     2880-byte block at a time...
      IF (idbg.GT.0) WRITE(*,*) dbid//'  writing XHU #',(3*n-1),
     +   ' to output MEF file...'
      j    = 0
      nblk = oblk
      oblk = oblk + 1
      CALL cinit(tmpblk)
      DO i = 1,nrec
         j = j + 1
         IF (j.LE.36) THEN
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ELSE
C           Write the 2880-byte block to the output MEF file...
            CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
            IF (ierr.NE.0) GOTO 9
C           Re-initialize the 2880-byte block buffer...
            j = 1
            oblk = oblk + 1
            CALL cinit(tmpblk)
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ENDIF
      ENDDO

C     Write the partially populated (remainder is blank) or exactly
C     filled 2880-byte block to the output MEF file...
      CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      WRITE(*,*)' wrote XHU #',(3*n-1),' to output MEF file (',
     +   (oblk-nblk),' blocks )'


C     Copy the dummy DQI extension in original STIS dataset MEF file...
      last = .FALSE.
      nrec = 0
6     iblk = iblk + 1
      CALL rjfghblk (STDIFIL, iblk, block, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C     Parse this block into 36 individual 80-char header records...
      DO i = 1,36
C        Read the next record from the 2880-byte block...
         IF (.NOT.last) THEN
           CALL rjfghrec (block, i, crec, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           nrec = nrec + 1
           IF (idbg.GT.1) WRITE(*,'(1x,(A),I3,(A))') dbid//'record #',
     +                  nrec,':  "'//crec(1:clen(crec))//'"'

C          Parse the keyword from the record; search for END...
           CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
           IF (ierr.NE.0) GOTO 9
           IF (ckey(1:j).EQ.'END') THEN
             IF (idbg.GT.0) WRITE(*,*) dbid//
     +         'detected END keyword; last = T'
             lcom = .FALSE.
             last = .TRUE.
           ELSE
C            Some other header keyword detected...
             CONTINUE
           ENDIF
           IF (ierr.NE.0) GOTO 9

C          Append the record read to the output header array...
           ohdr(nrec) = crec
         ENDIF
      ENDDO

C     Try and read the next 2880-byte block from the extension header...
      IF (.NOT.last) GOTO 6
      IF (MOD(nrec,36).EQ.0) nrec = nrec - 1
      IF (idbg.GT.0) WRITE(*,*) dbid//' --> XHU has ',nrec,' keywords.'

C     Write the dummy extension (just a HU) to the output MEF file, one
C     2880-byte block at a time...
      IF (idbg.GT.0) WRITE(*,*) dbid//'  writing XHU #',(3*n),
     +   ' to output MEF file...'
      j    = 0
      nblk = oblk
      oblk = oblk + 1
      CALL cinit(tmpblk)
      DO i = 1,nrec
         j = j + 1
         IF (j.LE.36) THEN
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ELSE
C           Write the 2880-byte block to the output MEF file...
            CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
            IF (ierr.NE.0) GOTO 9
C           Re-initialize the 2880-byte block buffer...
            j = 1
            oblk = oblk + 1
            CALL cinit(tmpblk)
C           Write the current record to the 2880-byte block...
            k = (j-1)*80 + 1
            m = j*80
            tmpblk(k:m) = ohdr(i)(1:80)
         ENDIF
      ENDDO

C     Write the partially populated (remainder is blank) or exactly
C     filled 2880-byte block to the output MEF file...
      CALL rjfphblk (tmpblk, STDOFIL, oblk, idbg, ierr)
      IF (ierr.NE.0) GOTO 9
      WRITE(*,*)' wrote XHU #',(3*n),' to output MEF file (',
     +   (oblk-nblk),' blocks )'


C     Continue with the next extension/flat FITS frame, if any...
      IF (n.LT.nf.AND.((3*n)-2).LT.nxtd) GOTO 2
C  -- End of loop over the extensions.

      IF (n.LT.nf.AND.((3*n)-2).GE.nxtd) THEN
         WRITE(*,*) dbid//' number of frames in input list exceeds '//
     +      'the number of SCI extensions in the original STIS dataset'
         ierr = 1
      ENDIF


C --- Close the open original STIS dataset MEF file...
      CALL rjf_close(STDIFIL, idbg, ierr)
      IF (ierr.NE.0) GOTO 9

C --- Close the open output MEF file...
      CALL rjf_close(STDOFIL, idbg, ierr)

9     IF (ierr.EQ.0) THEN
         WRITE(*,'(A)') 'FITS2MEF: finished.'
      ELSE IF (ierr.NE.-1) THEN
         WRITE(*,*)'FITS2MEF: failed! (ierr=',ierr,' on exit)'
      ENDIF

      STOP
      END



C ----------------------------------------------------------------------
C ++++++++  S U B R O U T I N E S   A N D   F U N C T I O N S  +++++++++
C ----------------------------------------------------------------------

      SUBROUTINE clinput (omef, olen, imef, ilen, tfil, tlen, idbg,ierr)

C @(#)Subroutine: clinput            Author: R.A. Jansen -- Apr  7, 2010
C @(#)
C @(#)  Purpose: Parse the command-line for  1) the name of the original
C @(#)    STIS data set (MEF format), possibly with a path name,  2) the
C @(#)    the name of the output MEF file to write, possibly with a path
C @(#)    name, 3) the name of an input text file containing one or more
C @(#)    lines of  comma-separated fields  of a  FITS extension number,
C @(#)    FITS file name,  parameters that characterize  the herringbone
C @(#)    pattern-noise detected,  and whether or not that was a genuine
C @(#)    pattern, and 4) an optional verbosity level modifier.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL clinput (omef,olen,imef,ilen,tfil,tlen,idbg,ierr) , where
C @(#)    'omef', 'imef', and 'ifil'  are character strings of arbitrary
C @(#)    declared lengths, and 'olen', 'ilen', and 'tlen'  are integers
C @(#)    returning their actual lengths.  If 'fits2mef' is called  with
C @(#)    optional debug switch "-d",  diagnostic messaging is turned on
C @(#)    (the program will become very verbose).  If it appear that the
C @(#)    calling MAIN routine ought to display the program's  man page,
C @(#)    the error status 'ierr' will have a value of -1 upon exit.
C @(#)
C @(#)  Note:  this subroutine  is specific to program "fits2mef", not a
C @(#)    generic one, and should *not* be included in a library.
C @(#)
C @(#)  Dependencies:     iargc and getarg (both 'libF77.a'/'libg2c.a'),
C @(#)                    cinit, clen, error
C @(#)
C @(#)  Revision history:
C @(#)    Apr  7, 2010 [RAJ] -- based on/excised from routines  "fixhdr"
C @(#)                          and "fitsheader" (v2.0).
C @(#)    Jun 29, 2010 [RAJ] -- changed calling sequence: added 'imef'.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER*(*) omef, imef, tfil
      INTEGER	olen, ilen, tlen, idbg, ierr

C --- Declaration of local variables...
      CHARACTER	ctmp*256, carg(3)*128, dbid*19
      INTEGER*4	narg, iargc
      INTEGER	clen, mlen, i, j


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the output variables...
      CALL cinit(omef)
      CALL cinit(imef)
      CALL cinit(tfil)
      idbg      =  0
      dbid      = 'DEBUG(clinput):    '

C --- Count the number of command-line arguments; there must be at least
C     one specified (if only one argument it must be the '-h' switch)...
      narg = iargc()
      IF (narg.LT.1) THEN
         CALL error('clinput','no command-line argument(s) specified')
         ierr = -1
         RETURN
      ENDIF

C --- Try and parse all command-line arguments and test whether any is a
C     help or debug/verbosity switch; keep others for further scrutiny..
      j = 0
      DO i=1,narg
         CALL cinit(ctmp)
         CALL getarg(i,ctmp)
         IF (ctmp(1:2).EQ.'-h'.OR.ctmp(1:3).EQ.'--h') THEN
            ierr = -1
            RETURN
         ELSE IF (ctmp(1:3).EQ.'-dd'.OR.ctmp(1:4).EQ.'--dd') THEN
            idbg = idbg + 2
            IF (idbg.GE.2) WRITE(*,*) dbid//'switched ON debug mode'
         ELSE IF (ctmp(1:2).EQ.'-d'.OR.ctmp(1:3).EQ.'--d') THEN
            idbg = idbg + 1
            IF (idbg.EQ.1) WRITE(*,*) dbid//'switched ON verbose mode'
            IF (idbg.EQ.2) WRITE(*,*) dbid//'switched ON debug mode'
         ELSE
            j = j + 1
            IF (j.GT.4) THEN
               CALL cinit(ctmp)
               ctmp = 'more command-line arguments than MAXARG = 4'
               CALL error('clinput',ctmp)
               ierr = -1
               RETURN
            ENDIF
            CALL cinit(carg(j))
            mlen = MIN(clen(ctmp),LEN(carg(j)))
            carg(j)(1:mlen) = ctmp(1:mlen)
         ENDIF
      ENDDO

      IF (j.LT.3) THEN
         CALL cinit(ctmp)
         ctmp = 'too few command-line arguments specified'
         CALL error('clinput',ctmp)
         ierr = -1
         RETURN
      ELSE IF (j.GT.3) THEN
         CALL cinit(ctmp)
         ctmp = 'too many arguments or incorrect syntax'
         CALL error('clinput',ctmp)
         ierr = -1
         RETURN
      ELSE IF (    carg(1)(1:1).EQ.'-'
     +         .OR.carg(2)(1:1).EQ.'-'
     +         .OR.carg(3)(1:1).EQ.'-') THEN
         CALL cinit(ctmp)
         ctmp = 'incorrect syntax or unrecognized option'
         CALL error('clinput',ctmp)
         ierr = -1
         RETURN
      ELSE
C        So far, so good...
         CONTINUE
      ENDIF

C --- When here, there was no cry for help and we have already filtered
C     out any debug/verbosity switch; the three and only three command
C     line arguments must be the names of the output MEF, original STIS
C     dataset, and input text files...

      mlen = MIN(clen(carg(1)),LEN(omef))
      IF (mlen.GT.0) THEN
         omef(1:mlen) = carg(1)(1:mlen)
C        Ignore any image section and/or extension specifiers...
         i = INDEX(omef,'[')
         IF (i.NE.0) THEN
            DO j = i,mlen
               omef(j:j) = ' '
            ENDDO
         ENDIF
         olen = clen(omef)
      ELSE
         CALL error('clinput','reading name of output MEF file')
         ierr = 1
         RETURN
      ENDIF

      mlen = MIN(clen(carg(2)),LEN(imef))
      IF (mlen.GT.0) THEN
         imef(1:mlen) = carg(2)(1:mlen)
C        Ignore any image section and/or extension specifiers...
         i = INDEX(imef,'[')
         IF (i.NE.0) THEN
            DO j = i,mlen
               imef(j:j) = ' '
            ENDDO
         ENDIF
         ilen = clen(imef)
      ELSE
         CALL error('clinput','reading name of original STIS dataset')
         ierr = 1
         RETURN
      ENDIF

      mlen = MIN(clen(carg(3)),LEN(tfil))
      IF (mlen.GT.0) THEN
         tfil(1:mlen) = carg(3)(1:mlen)
         tlen = clen(tfil)
      ELSE
         CALL error('clinput','reading name of input text file')
         ierr = 1
         RETURN
      ENDIF

      RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE flinput (tfil,tlen, nf,ifts,flen,iext,hparm, idbg,ierr)

C @(#)Subroutine: flinput             Author: R.A. Jansen -- Apr 15 2010
C @(#)
C @(#)  Purpose: parse the contents of text file 'tfil' and return 'nf'-
C @(#)    element vectors 'ifts','flen', and 'iext',  of FITS file names
C @(#)    and their actual (as opposed to declared) length, and the FITS
C @(#)    extension these correspond to in the original STIS dataset MEF
C @(#)    file,  and an ('nf',4)-matrix 'hparm' of triplets of parameter
C @(#)    values that characterize the pattern noise detected and a flag
C @(#)    that indicates whether or not that pattern was genuine (0.) or
C @(#)    not (>0).  The fields in each line of 'tfil' should be separa-
C @(#)    ted by either white space or commas. If verbosity level 'idbg'
C @(#)    >0,  additional messages may be printed.   Error status 'ierr'
C @(#)    will be  non-zero  upon exit,  should any  error condition  be
C @(#)    detected.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL flinput (tfil,tlen, nf,ifts,flen,iext,hparm, idbg,ierr) ,
C @(#)    with  'tfil'  a character string  of arbitrary declared length
C @(#)    specifying the  'tlen'-character  name of an  input text file,
C @(#)    'nf'  an integer that returns the number of lines successfully
C @(#)    parsed from 'tfil', 'ifts' an 'nf'-element character*80 valued
C @(#)    vector that returns the names of the input FITS files,  'flen'
C @(#)    an 'nf'-element integer-valued vector that returns the lengths
C @(#)    of those FITS file names, 'iext' a 'nf'-element integer-valued
C @(#)    vector that returns the FITS extension numbers that thes files
C @(#)    correspond to,  and  'hparm'  an ('nf',4)-element  real-valued
C @(#)    matrix that returns the pattern-noise parameters and validity.
C @(#)    'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note:  this subroutine  is specific to program "fits2mef", not a
C @(#)    generic one, and should *not* be included in a library.
C @(#)
C @(#)  Dependencies:     opendev, cinit, clen, error, parse, creadr;
C @(#)                    'devices.inc', 'buffers.inc'
C @(#)
C @(#)  Revision history:
C @(#)    Apr 15, 2010 [RAJ] -- Initial version of this routine.
C @(#)    Jul  2, 2010 [RAJ] -- Added 'iext' to calling sequence.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Learn about maximum image dimension, maximum axis lengths, maximum
C     number of FITS extensions, and maximum number of header records...
      INCLUDE	'buffers.inc'

C --- Declaration of input/return variables...
      CHARACTER*(*) tfil
      CHARACTER*80 ifts(MAXEXT), cfil
      REAL	hparm(MAXEXT,4)
      INTEGER	tlen, nf, flen(MAXEXT), iext(MAXEXT), idbg, ierr

C --- Declaration of local variables...
      CHARACTER	cline*128, ctmp*80, dbid*19, cc*1
      REAL	rval
      INTEGER	ival, clen, mlen, i, j


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the output variables...
      nf        = 0
      cc        = CHAR(32)
      dbid      = 'DEBUG(flinput):    '


C --- Try and open the input text file for reading...
      CALL cinit(cfil)
      j = MIN(tlen,LEN(cfil))
      cfil(1:j) = tfil(1:j)
      IF (idbg.GT.0) WRITE(*,*)
     +   dbid//'calling "opendev" (RO), file="'//cfil(1:j)//'"'
      CALL opendev(STDIFIL, cfil, 'read', 'sequential', 128, ierr)
      IF (ierr.NE.0) RETURN

C --- Try and parse the input text file, one line at a time...
1     CALL cinit(cline)
      READ (STDIFIL,'(A)',END=2) cline

      mlen = clen(cline)
      IF (mlen.LE.0) THEN
         CALL cinit(ctmp)
         ctmp = 'read an unexpected blank line'
         CALL error('flinput',ctmp)
         ierr = 1
         RETURN
      ELSE
C        Increment the number of lines read...
         nf = nf + 1

C        Initialize file name, and noise parameter values...
         ifts(nf) = CHAR(32)
         flen(nf) = 0
         iext(nf) = 0
         hparm(nf,1) = 0.00
         hparm(nf,2) = 0.00
         hparm(nf,3) = 0.00
         hparm(nf,4) = 0.00

C        Try and parse a FITS extension number from 'cline'...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         ival = 0
         CALL creadi(ctmp,ival,ierr)
         IF (ierr.NE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse FITS extension number'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (ival.LT.1.OR.ival.GT.MAXEXT) THEN
            CALL cinit(ctmp)
            ctmp = 'FITS extension number out of valid range 1..MAXEXT'
            CALL error('flinput',ctmp)
            ierr =1
         ELSE
            iext(nf) = ival
         ENDIF
         IF (ierr.NE.0) RETURN

C        Try and parse a FITS file name from 'cline'...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         IF (i.LE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse FITS file name'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (i.GT.80) THEN
            CALL cinit(ctmp)
            ctmp = 'FITS file name exceeds 80 characters in length'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE
            flen(nf) = i
            ifts(nf)(1:i) = ctmp(1:i)
         ENDIF
         IF (ierr.NE.0) RETURN

C        Try and parse a frequency from the remainder of 'cline'...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         rval = 0.00
         CALL creadr(ctmp,rval,ierr)
         IF (ierr.NE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse a pattern frequency'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (rval.LT.10000.0.OR.rval.GT.20000.0) THEN
            CALL cinit(ctmp)
            ctmp = 'unexpected value outside of range 10000--20000 Hz'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE
            hparm(nf,1) = rval
         ENDIF
         IF (ierr.NE.0) RETURN

C        Try and parse a drift-width from the remainder of 'cline'...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         rval = 0.00
         CALL creadr(ctmp,rval,ierr)
         IF (ierr.NE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse a pattern drift-width'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (rval.LT.0.0.OR.rval.GT.100.0) THEN
            CALL cinit(ctmp)
            ctmp = 'unexpected value outside of range 0--100 Hz'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE
            hparm(nf,2) = rval
         ENDIF
         IF (ierr.NE.0) RETURN

C        Try and parse an amplitude from the remainder of 'cline'...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         rval = 0.00
         CALL creadr(ctmp,rval,ierr)
         IF (ierr.NE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse a pattern power amplitude'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (rval.LT.-0.1.OR.rval.GT.1.1) THEN
            CALL cinit(ctmp)
            ctmp = 'unexpected value outside of range ~0.0--1.0'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE
            hparm(nf,3) = rval
         ENDIF
         IF (ierr.NE.0) RETURN

C        Try and parse an success/failure flag from the remainder...
         CALL cinit(ctmp)
         CALL parse(cline,cc,ctmp,i)
         rval = -1.00
         CALL creadr(ctmp,rval,ierr)
         IF (ierr.NE.0) THEN
            CALL cinit(ctmp)
            ctmp = 'failed to parse a success/failure flag'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE IF (rval.LT.-1.0.OR.rval.GT.1.0) THEN
            CALL cinit(ctmp)
            ctmp = 'unexpected flag value outside of range -1.0--+1.0'
            CALL error('flinput',ctmp)
            ierr = 1
         ELSE
            hparm(nf,4) = rval
         ENDIF
         IF (ierr.NE.0) RETURN

      ENDIF

C     Report our progress...
      IF (idbg.GT.0) WRITE(*,*) dbid//'read line #',nf,' ...'
      IF (idbg.GT.0) THEN
         WRITE(*,*)dbid//'  clen(line)=',mlen,', flen(',nf,')=',flen(nf)
         WRITE(*,'(1x,A28,I03,(A))') dbid//'    image #',
     +           nf,' ="'//ifts(nf)(1:flen(nf))//'"'
         WRITE(*,'(1x,A34,F7.1,A1,F6.4,A1,F7.5,A1,F3.1,A1)')
     +           dbid//'    (f,w,p,q)=(',hparm(nf,1),',',hparm(nf,2),
     +           ',',hparm(nf,3),',',hparm(nf,4),')'
      ENDIF

C --- Try and read the next line...
      GOTO 1


C --- Close the input text file...
2     IF (nf.EQ.0) THEN
         CALL cinit(ctmp)
         ctmp = 'the input text file is empty!'
         CALL error('flinput',ctmp)
      ELSE
         IF (idbg.GT.0) WRITE(*,*) dbid//'EOF detected; closing file...'
      ENDIF
      CLOSE(UNIT=STDIFIL,STATUS='KEEP',IOSTAT=ierr)
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'error closing file '//
     +      cfil(1:j)//' attached to logical device unit ',STDIFIL
         ierr = 1
      ENDIF
      IF (nf.EQ.0) ierr = 1


9     RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE manual()

C @(#)Subroutine: manual             Author: R.A. Jansen -- Jul 30, 2008
C @(#)  
C @(#)  Purpose: Write a short manual page showing the proper syntax and
C @(#)    command-line options to program 'fits2mef'.
C @(#)
C @(#)  Calling sequence: CALL manual()
C @(#)
C @(#)  Dependencies:  clen, error, fwritc; 'devices.inc'
C @(#)
C @(#)  Note: the path to the man-page text file  is presently hardcoded
C @(#)    and should be changed or replaced with a global manpath and
C @(#)    file name when porting this routine.
C @(#)
C @(#)  Revision history:
C @(#)    Jul 31, 2008 [RAJ] -- added internal error status variable due
C @(#)                          to changed calling sequence of 'fwritc'.
C @(#)    Apr  7, 2010 [RAJ] -- modified Purpose and the manual filename
C @(#)                          as appropriate for 'fits2mef'.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE   'devices.inc'

C --- Declaration of input variables...
C     (none)

C --- Declaration of local variables...
      CHARACTER*(80) mfile, cline
      INTEGER   iunit, clen, ierr
      LOGICAL   ok, busy

C --- Initialize...
      ierr = 0

C --- Name of text file containing the manual...
      mfile = 'fits2mef.man'

C --- Check whether the manual file is indeed present...
      INQUIRE(FILE=mfile(1:clen(mfile)),EXIST=ok)
      IF (.NOT.ok) THEN
         cline = 'ERROR: '//mfile(1:clen(mfile))//' not found!'
         CALL fwritc(STDOUT,cline,ierr)
         RETURN
      ENDIF

C --- Find an available file unit number...
      DO iunit=STDOFIL,99
         INQUIRE (UNIT=iunit,OPENED=busy)
         IF (.NOT.busy) GOTO 1
      ENDDO
      CALL error('manual','could not find any free logical device unit')

C --- Open the manual file for reading...
1     OPEN(UNIT=iunit,FILE=mfile(1:clen(mfile)),STATUS='OLD',ERR=9)

C --- Print the manual page...
2     READ(iunit,'(A)',END=3) cline
      CALL fwritc(STDOUT,cline,ierr)
      GOTO 2

C --- Close the manual file...
3     CLOSE(UNIT=iunit)

9     RETURN
      END


C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE fwritc(iunit,cstr,ierr)

C @(#)Subroutine: fwritc             Author: R.A. Jansen -- Feb 12, 1998
C @(#)
C @(#)  Purpose: Write a character string 'cstr' to the specified device
C @(#)    unit 'iunit'.
C @(#)
C @(#)  Calling sequence: CALL fwritc(iunit,cstr,ierr)   , where 'iunit'
C @(#)    must be open for writing already,  and  'cstr'  is a character
C @(#)    string of arbitrary declared length, of which only the portion
C @(#)    up to the actual length will be printed. If an error condition
C @(#)    is triggered, 'ierr'  will be non-zero upon exit.
C @(#)
C @(#)  Dependencies:     clen
C @(#)
C @(#)  Revision history:
C @(#)    May 22, 2008 [RAJ] -- expanded documentation.
C @(#)    Jul 31, 2008 [RAJ] -- added error status parameter 'ierr', for
C @(#)                          when 'iunit' isn't the standard output. 
C @(#)------------------------------------------------------------------

      CHARACTER*(*) cstr
      INTEGER   iunit, ierr, clen

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Write input character string to output device... 
      WRITE(iunit,'(A)',IOSTAT=ierr) cstr(1:clen(cstr))

      RETURN
      END

C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
