C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfvk_pq (dbid,ri,ctmp,ii,jj, cval,nd,lexp, idbg,ierr)

C @(#)Subroutine: rjfvk_pq           Author: R.A. Jansen -- Oct 13, 2009
C @(#)
C @(#)  Purpose: Perform the character validity tests that are common to
C @(#)    the 'rjfgkval[pq]' routines  to read a complex-valued  keyword
C @(#)    value from a sub-string 'ctmp' of a FITS header record.  Input
C @(#)    character string 'dbid' identifies the calling routine,  which
C @(#)    each calls the present routine twice: for the real ('ri' = 0),
C @(#)    and for the imaginary ('ri' = 1)  part of a  complex floating-
C @(#)    point value.  Input character string 'ctmp' contains both, but
C @(#)    variables 'ii' and 'jj' specify the position of  the first and
C @(#)    last character of the sub-string that represents  the floating
C @(#)    point value of interest.   Upon exit, character string  'cval'
C @(#)    will contain that verified sub-string. Also upon exit, integer
C @(#)    'nd'  will return  the number of digits in the mantissa of the
C @(#)    floating point value, and boolean flag 'lexp' will be returned
C @(#)    true if the floating point value extracted from 'ctmp'  was in
C @(#)    exponential notation.  If verbosity level 'idbg'>0, additional
C @(#)    messages may be printed.  Error status 'ierr' will be non-zero
C @(#)    upon exit, should any of the tests/checks fail.  Its value (if
C @(#)    >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfvk_pq (dbid,ri,ctmp,ii,jj, cval, nd, lexp, idbg,ierr),
C @(#)    where 'dbid' is a character string with declared length of  19
C @(#)    characters; 'ri' is an integer restricted to the value 0 or 1;
C @(#)    'ctmp' and 'cval'  are character strings with declared lengths
C @(#)    of 80 characters;  'ii', 'jj', 'nd', 'idbg' and 'ierr' are all
C @(#)    integers; and 'lexp' is a boolean flag.
C @(#)
C @(#)  Note:  this auxilliary routine is meant to be called only by the
C @(#)    'rjfgkval[pq]'  routines,  not directly by the user.  Its only
C @(#)    purpose is to collect common code into a single routine.
C @(#)  Note2: 'ii' must be the position of the first *non-blank* charac
C @(#)    ter of the substring representing the floating point value.
C @(#)  Note3: this auxilliary routine  may also be useful  for routines
C @(#)    'rjfgkval[ed]' with 'ri'=2 (undocumented but implemented).
C @(#)
C @(#)  Dependencies:   clen, cinit
C @(#)
C @(#)  Revision history:
C @(#)    Oct 13, 2009 [RAJ] -- Initial version; code extracted from the
C @(#)                          'rjfgkval[pq]' routines.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE
      
C --- Declaration of input/return variables...
      CHARACTER	dbid*19, ctmp*80, cval*80
      INTEGER   ri, ii, jj, nd, idbg, ierr

C --- Declaration of local variables...
      CHARACTER	cc, sstr*24, cstr*48
      INTEGER	clen, ic, k, m, n

C     The following variables are shorthand integer equivalents in the
C     ASCII collating sequence of space, digits 0 and 9, E, D, plus and
C     minus, and decimal point, respectively...
      INTEGER   ib, i0, i9, iEE, iDD, ip, im, id

C     Flags 'lsgn', 'lexp' and 'ldp' are raised when a + or - sign, an
C     exponent character, and a decimal point are found, respectively;
C     'last' is raised when a (terminating) space character is found...
      LOGICAL   lsgn, lexp, ldp, last

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the return values...
      CALL cinit(cval)
      lexp = .FALSE.
      nd   = 0

C --- Initialize local variables...
      ib  = ICHAR(' ')
      i0  = ICHAR('0')
      i9  = ICHAR('9')
      iEE = ICHAR('E')
      iDD = ICHAR('D')
      ip  = ICHAR('+')
      im  = ICHAR('-')
      id  = ICHAR('.')
      lsgn = .FALSE.
      ldp  = .FALSE.
      last = .FALSE.

C     Identifying string for messages...
      CALL cinit(sstr)
      CALL cinit(cstr)
      IF (ri.EQ.0) sstr(1:12) = 'real part of'
      IF (ri.EQ.1) sstr(1:17) = 'imaginary part of'
      k = clen(sstr)
      IF (k.GT.0) THEN
         cstr(1:(19+k)) = dbid(1:19)//sstr(1:k)
      ELSE
         cstr(1:19) = dbid(1:19)
      ENDIF
      k = k + 19


C --- Test that the first non-blank character in the floating point
C     value is a sign, a digit, or a decimal point...
      cc = ctmp(ii:ii)
      ic = ICHAR(cc)
      IF (ic.NE.ip .AND. ic.NE.im .AND. ic.NE.id
     +    .AND. (ic.LT.i0 .OR. ic.GT.i9)         ) THEN
C        Unexpected first non-blank char; not a real value!
         IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value is '//
     +      'not a real (does not start with a sign, digit, or '//
     +      'decimal point)'
         ierr = 2721
         IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         GOTO 9
      ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C        First non-blank character is a sign character...
         IF (idbg.GT.2) WRITE(*,*) cstr(1:k)//' keyword value '//
     +      'starts with a sign'
         lsgn = .TRUE.
      ELSE IF (ic.GE.i0 .AND. ic.LE.i9) THEN
C        First non-blank character is a digit...
         IF (idbg.GT.2) WRITE(*,*) cstr(1:k)//' keyword value '//
     +      'starts with a digit'
         nd = nd + 1
      ELSE IF (ic.EQ.id) THEN
C        First non-blank character is a decimal point...
         IF (idbg.GT.2) WRITE(*,*) cstr(1:k)//' keyword value '//
     +      'starts with a decimal point'
         ldp = .TRUE.
      ENDIF


C --- Copy the non-blank characters of the numeric keyword value from
C     'ctmp' to temporary value string 'cval', one character at a time,
C     while testing on-the-fly for any illegal character that is incon-
C     sistent with a floating point value. The first character in 'cval'
C     will be blank to aid in list-directed reads...
      m = 2
      cval(m:m) = cc
      DO n = (ii+1),jj,1

         cc = ctmp(n:n)
         ic = ICHAR(cc)

C        Test whether this is a valid character...
         IF (.NOT.last .AND. ic.EQ.ib) THEN
C           We must have read all characters representing the real...
            last = .TRUE.
         ELSE IF (last .AND. ic.EQ.ib) THEN
C           Trailing space following characters representing the real...
            CONTINUE
         ELSE IF (last .AND. ic.NE.ib) THEN
C           Detected an embedded blank character: value is not a real...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains embedded space character'
            ierr = 2724
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ELSE IF (.NOT.lexp .AND. (ic.GE.i0 .AND. ic.LE.i9) ) THEN
C           Valid character (digit in mantissa), so copy it to 'cval'...
            m = m + 1
            cval(m:m) = cc
            nd = nd + 1
         ELSE IF (lexp .AND. (ic.GE.i0 .AND. ic.LE.i9)) THEN
C           Valid character (digit in exponent), so copy it to 'cval'...
            m = m + 1
            cval(m:m) = cc
         ELSE IF (.NOT.ldp .AND. ic.EQ.id) THEN
C           Valid character (decimal point in mantissa), copy to 'cval'.
            m = m + 1
            cval(m:m) = cc
            ldp = .TRUE.
         ELSE IF (ldp .AND. ic.EQ.id) THEN
C           More than one decimal point detected: value is not a real...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains more than one decimal point'
            ierr = 2727
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ELSE IF (.NOT.lexp .AND. (ic.EQ.iEE .OR. ic.EQ.iDD) ) THEN
C           Valid character (exponent sign); reset sign flag; change 'E'
C           to 'D' to aid in list-directed read...
            m = m + 1
            cval(m:m) = 'D'
            lexp = .TRUE.
            lsgn = .FALSE.
         ELSE IF (lexp .AND. (ic.EQ.iEE .OR. ic.EQ.iDD) ) THEN
C           Detected more than one exponent: value is not a real...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains more than one exponent character'
            ierr = 2730
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ELSE IF (lexp .AND. lsgn .AND. (ic.EQ.ip .OR. ic.EQ.im)) THEN
C           More than one sign in exponent: value is not a real...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains more than one sign in the exponent'
            ierr = 2733
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ELSE IF (lsgn .AND. (ic.EQ.ip .OR. ic.EQ.im)) THEN
C           More than one sign in mantissa: value is not a real...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains more than one sign in the mantissa'
            ierr = 2736
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ELSE IF (.NOT.lsgn .AND. (ic.EQ.ip .OR. ic.EQ.im)) THEN
C           Valid character (sign in exponent), so copy it to 'cval'...
            m = m + 1
            cval(m:m) = cc
            lsgn = .TRUE.
         ELSE
C           Illegal character: neither sign, digit, decimal point, nor
C           exponent character...
            IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value '//
     +         'contains character other than sign, decimal point, '//
     +         'digit, or exponent character'
            ierr = 2739
            IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
         ENDIF
         IF (ierr.NE.0) GOTO 9

      ENDDO

C --- Append "d0" to values that are not in exponential notation...
      IF (.NOT.lexp) THEN
         m = m + 2   
         cval((m-1):m) = 'D0'
      ENDIF
      cval((m+1):(m+1)) = ' '


C --- Test that the mantissa contained at least one digit, and that the
C     final character of any exponent is a digit...
      cc = cval(m:m)
      ic = ICHAR(cc)
      IF (nd.LT.1) THEN
         IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value is '//
     +      'malformatted (no digits in the mantissa)'
         ierr = 2742
         IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
      ENDIF
      IF (lexp .AND. (ic.LT.i0 .OR. ic.GT.i9)) THEN
         IF (idbg.GT.0) WRITE(*,*) cstr(1:k)//' keyword value is '//
     +      'malformatted (no digits in the exponent)'
         ierr = 2745
         IF (ri.EQ.0 .OR. ri.EQ.1) ierr = ierr + ri + 1
      ENDIF
      IF (ierr.NE.0) GOTO 9


9     RETURN
      END


