C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfvkmpq (dbid, crec, ctmp, i, j, k, idbg, ierr)

C @(#)Subroutine: rjfvkmpq           Author: R.A. Jansen -- Oct 13, 2009
C @(#)
C @(#)  Purpose: Perform the preparatory tests/checks that are common to
C @(#)    the 'rjfgkval[mpq]' routines  to read a complex-valued keyword
C @(#)    value from a FITS header record 'crec'. Input character string
C @(#)    'dbid'  identifies the calling routine.  Upon exit,  character
C @(#)    string 'ctmp' contains the nominal value portion of the header
C @(#)    record,  'i' returns the byte position in 'ctmp'  of the first
C @(#)    non-blank character following the opening parenthesis, 'j' the
C @(#)    position of the comma that separates real and imaginary parts,
C @(#)    and 'k' the position of the last non-blank character preceding
C @(#)    the closing parenthesis. If verbosity level 'idbg'>0, additio-
C @(#)    messages may be printed.  Error status 'ierr' will be non-zero
C @(#)    upon exit, should any of the tests/checks fail.  Its value (if
C @(#)    >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfvkmpq (dbid, crec, ctmp, i, j, k, idbg, ierr),   where
C @(#)    'dbid' is a character string with a declared length of 19 cha-
C @(#)    racters; 'crec' and 'ctmp' are character strings with declared
C @(#)    lengths of 80 characters; 'i', 'j', 'k', 'idbg' and 'ierr' are
C @(#)    all integers.
C @(#)
C @(#)  Note:  this auxilliary routine is meant to be called only by the
C @(#)    'rjfgkval[mpq]'  routines, not directly by the user.  Its only
C @(#)    purpose is to collect common code into a single routine.
C @(#)
C @(#)  Dependencies:   clen, cidx, lenc, ucase
C @(#)
C @(#)  Revision history:
C @(#)    Oct 13, 2009 [RAJ] -- Initial version; code extracted from the
C @(#)                          'rjfgkval[mpq]' routines.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE
      
C --- Declaration of input/return variables...
      CHARACTER crec*80, ctmp*80, dbid*19
      INTEGER   i, j, k, idbg, ierr

C --- Declaration of local variables...
      CHARACTER	cc, po, pc
      INTEGER	clen, lenc, cidx, ic, ie, is, m

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the return index values...
      i = 80
      j = 0
      k = 0

C --- Initialize local variables...
      ie = ICHAR('=')
      is = ICHAR('/')

C --- Test that byte 9 in 'crec' is indeed an equals sign (captures user
C     error of calling 'rjfgkval[mpq]' before 'rjfvkval') ...
      cc = crec(9:9)
      ic = ICHAR(cc)
      IF (ic.NE.ie) THEN
C        Not an equals-sign, therefore no keyword value in this record!
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'header record does not contain a keyword value'
         ierr = 2711
         GOTO 9
      ENDIF

C --- Assign the nominal value portion of the header record to 'ctmp',
C     and convert any alphabetic characters to upper case.  In v3.0 of
C     the FITS standard free-format values are allowed to start anywhere
C     after byte 10...
      ctmp(1:71)  = crec(10:80)
      ctmp(72:80) = '         '
      CALL ucase(ctmp,ctmp)

C --- Test that the first non-blank character in the value portion of
C     the header record was indeed an opening parenthesis (captures the
C     user error of calling 'rjfgkval[mpq]' while return value 'ityp' of
C     'rjfvkval' was other than 8, 9 or 10) ...
      po = '('
      i = lenc(ctmp)
      k = cidx(ctmp,po)
      IF (i.NE.k) THEN
C        First non-blank character not a "("; keyword value not complex!
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not '//
     +      'complex (does not start with a parenthesis)'
         ierr = 2712
         GOTO 9
      ENDIF

C --- Find positions of the comma separating real and imaginary parts of
C     the complex keyword value, and of the closing parenthesis...
      cc = ','
      pc = ')'
      j = cidx(ctmp,cc)
      k = cidx(ctmp,pc)
      IF (j.EQ.0 .OR. k.EQ.0 .OR. k.LT.j) THEN
C        Malformatted complex value or not a complex value at all...
         IF (j.EQ.0 .AND. k.EQ.0) THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not '//
     +         'complex (missing comma and closing parenthesis)'
            ierr = 2713
         ELSE IF (j.EQ.0) THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not '//
     +         'complex (missing comma)'
            ierr = 2714
         ELSE IF (k.EQ.0) THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not '//
     +         'complex (missing closing parenthesis)'
            ierr = 2715
         ELSE IF (k.LT.j) THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not '//
     +         'complex (found closing parenthesis before comma)'
            ierr = 2716
         ENDIF
      ENDIF
      IF (ierr.NE.0) GOTO 9

C --- The first non-blank character, if any, following the closing
C     parenthesis must be a '/' ...
      m = lenc(ctmp((k+1):71))
      IF (m.GT.0) THEN 
         cc = ctmp((k+m):(k+m))
         ic = ICHAR(cc)
         IF (ic.NE.is) THEN
C           Detected a non-blank character other than a / ...
            IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword val'//
     +         'ue or header record: junk follows closing parenthesis'
            ierr = 2717
            GOTO 9
         ENDIF
      ENDIF

C --- Strip opening and closing parenthesis, replace comma with space,
C     and blank any non-blank characters beyond closing parenthesis...
      ctmp(i:i) = ' '
      ctmp(j:j) = ' '
      DO m = k,clen(ctmp),1
         ctmp(m:m) = ' '
      ENDDO
      i = lenc(ctmp)
      k = clen(ctmp)
            
      IF ( k.EQ.0 ) THEN
C        Illegal blank real and imaginary part of complex keyword value
         IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword value:'//
     +      ' real and imaginary parts of complex value are both blank'
         ierr = 2718
      ELSE IF ( i.GT.(j-1) ) THEN  
C        Illegal blank real part of complex keyword value...
         IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword value:'//
     +      ' real part of complex value is blank'
         ierr = 2719   
      ELSE IF ( k.LT.(j+1) ) THEN
C        Illegal blank imaginary part of complex keyword value...
         IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword value:'//
     +      ' imaginary part of complex value is blank'
         ierr = 2720   
      ENDIF
      IF (ierr.NE.0) GOTO 9


9     RETURN
      END


